#!/usr/bin/env python

from lxml import etree
import configparser
import os
import subprocess
import time
import zmq

MAC_ADDRESSES_COUNT = 2
EEPROM = "/sys/bus/spi/devices/spi0.0/eeprom"
LEGACY_EEPROM = "/sys/bus/spi/devices/spi32766.0/eeprom"


class ProductionClient:
    def __init__(self, address, device_type, mac_amount):
        self.__device_type = device_type
        self.__address = address
        self.__mac_amount = mac_amount

    def receive_setup(self):
        context = zmq.Context()
        socket = context.socket(zmq.REQ)
        socket.setsockopt(zmq.LINGER, 0)
        socket.setsockopt(zmq.RCVTIMEO, 10000)
        socket.connect(self.__address)

        socket.send_string("<request-bootstrap device=\"{0}\" mac-amount=\"2\"/>".format(self.__device_type))
        reply = socket.recv_string()
        socket.close()
        context.destroy()

        return self.parse(reply)

    @staticmethod
    def parse(message):
        tree = etree.XML(message)

        model_number = tree.xpath('/reply-bootstrap')[0].get('model-number')
        serial_number = tree.xpath('/reply-bootstrap')[0].get('serial-number')
        mac_addresses = []

        for node in tree.xpath('/reply-bootstrap/mac'):
            mac_addresses.append(node.get('address'))

        return model_number, serial_number, mac_addresses


class Settings:
    def __init__(self, path):
        self.__load(path)

    def __load(self, path):
        parser = configparser.ConfigParser()
        parser.read(path)

        self.__server_address = parser.get('server', 'production')
        self.__board_type = parser.get('board', 'type')

    def get_server_address(self):
        return self.__server_address

    def get_board_type(self):
        return self.__board_type


def wait_until_the_card_is_removed():
    while (not os.path.exists("/dev/mmcblk0boot0")) or (not os.path.exists("/dev/mmcblk0boot1")):
        print("INFO: Waiting, when user will remove SD card. It is needed to gain access to eMMC.")
        time.sleep(1)


def prepare_arguments(board_type, mac_addresses):
    arguments = " --board-type {0}".format(board_type)

    for mac_address in mac_addresses:
        arguments += " --mac {0}".format(mac_address)

    return arguments


def get_eeprom_name(model_number):
    matchbox_filename_lookup = {
        "11": "1228-X11X.bin",
        "12": "1228-X12X.bin",
        "21": "1228-X21X.bin",
        "22": "1228-X22X.bin",
        "41": "1228-X41X.bin",
        "42": "1228-X42X.bin",
        "51": "1228-X51X.bin",
        "52": "1228-X52X.bin",
    }
    diamond_filename_lookup = {
        "0": "240W-X0Z.bin",
        "1": "240W-X1Z.bin",
        "2": "240W-X2Z.bin",
        "3": "240W-X3Z.bin",
        "5": "240W-X5Z.bin",
    }

    if model_number[0:3] == "240":
        return diamond_filename_lookup[model_number[6:7]] # diamond models
    else:
        return matchbox_filename_lookup[model_number[6:8]] # matchbox models


def write_eeprom_image(file_path):
    subprocess.call("modprobe at25", shell=True)
    eeprom = EEPROM if os.path.exists(EEPROM) else LEGACY_EEPROM
    rc = subprocess.call("dd if={0} of={1}".format(file_path, eeprom), shell=True)

    if rc != 0:
        raise Exception("Can't write EEPROM")


def write_eeprom(file_path, address, data):
    with open(file_path, "rb+") as f:
        f.seek(address)
        f.write(data)


def bulk_write_eeprom(file_path, addresses, data):
    for address in addresses:
        write_eeprom(file_path, address, data)


def prepare_block(string, size):
    while len(string) < size:
        string += "\0"

    return bytearray(string.encode("ascii"))


def override_value(file_path, value, hi_addresses, lo_addresses):
    field_size = 8

    hi = prepare_block(value[:field_size], field_size)
    lo = prepare_block(value[field_size:field_size * 2], field_size)

    bulk_write_eeprom(file_path, hi_addresses, hi)
    bulk_write_eeprom(file_path, lo_addresses, lo)


def execute_emmc_script(script, board_type, mac_addresses):
    os.chmod(script, 0o744)
    os.system(script + prepare_arguments(board_type, mac_addresses))


def main():
    settings = Settings("/etc/bootstrap.config")
    production = ProductionClient(settings.get_server_address(), settings.get_board_type(), MAC_ADDRESSES_COUNT)
    model_number, serial_number, mac_addresses = production.receive_setup()
    print("INFO: Received model:", model_number, "S/N:", serial_number)

    if len(mac_addresses) != MAC_ADDRESSES_COUNT:
        print("FAILURE: Received wrong amount of mac addresses")
        return

    wait_until_the_card_is_removed()

    eeprom_path = os.path.join("/tmp", get_eeprom_name(model_number))
    override_value(eeprom_path, serial_number, [891, 899, 907], [916, 924, 932])
    override_value(eeprom_path, model_number, [941, 949, 957], [966, 974, 982])

    print("INFO: Writing eeprom image:", eeprom_path, "\n")
    write_eeprom_image(eeprom_path)

    execute_emmc_script("/tmp/prepare_emmc.sh", settings.get_board_type(), mac_addresses)

    print("SUCCESS: Device will be rebooted")
    os.system("reboot")


if __name__ == '__main__':
    main()
